SystemOrganization addCategory: #'Math-Tests-DHB-Numerical'!

TestCase subclass: #DhbDecimalFloatingNumberTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbDecimalFloatingNumberTest commentStamp: '<historical>' prior: 0!
A DhbDecimalFloatingNumberTest is a test class for testing the behavior of DhbDecimalFloatingNumber!

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 08:15'!
testAdd
	| a b sum |
	a := DhbDecimalFloatingNumber new: 0.10101.
	b := DhbDecimalFloatingNumber new: 0.2.
	sum := a + b.
	self should: [sum isKindOf: DhbDecimalFloatingNumber].
	self should: [sum value = (b + a) value].
	self should: [sum value isKindOf: Fraction ].
	self should: [sum value asFloat = 0.30101].! !

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 08:23'!
testDigits
	self should: [DhbDecimalFloatingNumber digits = DhbDecimalFloatingNumber defaultDigits].
	DhbDecimalFloatingNumber resetDigits.
	self shouldnt: [DhbDecimalFloatingNumber digits isNil].
	DhbDecimalFloatingNumber defaultDigits: 24.
	self shouldnt: [DhbDecimalFloatingNumber digits isNil].
	self should: [DhbDecimalFloatingNumber digits = 24].
	self should: [DhbDecimalFloatingNumber defaultDigits = 15].
	DhbDecimalFloatingNumber resetDigits.! !

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 08:36'!
testDisplay
	| a |
	a := DhbDecimalFloatingNumber new: 1.
	self should: [(a printString subStrings: 'E') last asInteger  = a class digits negated].
	
	a := DhbDecimalFloatingNumber new: 0.1.
	self should: [(a printString subStrings: 'E') last asInteger  = (a class digits negated - 1)].! !

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 09:12'!
testDivide
	| a b |
	a := DhbDecimalFloatingNumber new: 0.10101.
	b := DhbDecimalFloatingNumber new: 0.2.
	self should: [(a / b) isKindOf: DhbDecimalFloatingNumber].
	self should: [(a / b) value asFloat = 0.50505].
	self should: [(b / b) value = 1].
! !

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 08:10'!
testMultiply
	| a b |
	a := DhbDecimalFloatingNumber new: 0.10101.
	b := DhbDecimalFloatingNumber new: 0.2.
	self should: [(a * b) isKindOf: DhbDecimalFloatingNumber].
	self should: [(a * b) value asFloat = 0.020202].
	self should: [(b * b) value = (1/25)].
	self should: [(b * b) value asFloat = 0.04].! !

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 08:34'!
testRoundingErrors
	| a b |
	a := DhbDecimalFloatingNumber new: (2 raisedTo: 64).
	b := DhbDecimalFloatingNumber new: 300.
	self should: [(a + b) value = a value].! !

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 09:11'!
testSqrt
	| a  aSqrt |
	a := DhbDecimalFloatingNumber new: 0.01.
	aSqrt := a sqrt.
	self should: [aSqrt value asFloat = a value asFloat sqrt].
	self should: [aSqrt value = (1/10)].! !

!DhbDecimalFloatingNumberTest methodsFor: 'tests' stamp: 'DanielUber 6/4/2012 09:08'!
testSubtract
	| a b difference |
	a := DhbDecimalFloatingNumber new: 0.1.
	b := DhbDecimalFloatingNumber new: 0.2.
	difference := b - a.
	self should: [difference isKindOf: DhbDecimalFloatingNumber].
	
	" this errors raised by ln (domain is x > 0)"
	self should: [difference value ~= (a - b) value] raise: Error.
	self should: [difference value = (a - b) value negated] raise: Error.
	
	self should: [difference value isKindOf: Fraction ].
	self should: [difference value asFloat = 0.1].! !

TestCase subclass: #DhbFloatingPointMachineTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecision
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach machinePrecision > 0].
self should: [mach machinePrecision < 1].
self should: [mach negativeMachinePrecision > 0].
self should: [mach negativeMachinePrecision < 1].
self should: [(mach radix raisedTo: mach negativeMachinePrecision) = 1.0].
self should: [(mach radix raisedTo: mach machinePrecision) = 1.0].! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecisionIsFloatPrecision
"sanity check. Take logs to find the exponent, then compare to Float's method"

| prec |
prec := (DhbFloatingPointMachine new machinePrecision ln / DhbFloatingPointMachine new radix ln ) negated .
self should: [prec = Float precision].! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecisionLargestNumberGreaterThanAnyFractionOfItself
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach largestNumber * (1 - mach defaultNumericalPrecision) < mach largestNumber].
! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecisionLargestNumberIsLargest
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach largestNumber * (1 + mach defaultNumericalPrecision) = Float infinity].
! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecisionLargestNumberNotInfinity
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach largestNumber < Float infinity].! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecisionSmallNumberLargerThanSmallestNumber
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach smallestNumber < mach smallNumber].! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecisionSmallNumberNotZero
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach smallNumber > 0.0].! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testMachinePrecisionSmallestNumberNotZero
| mach |
mach := DhbFloatingPointMachine new.
self should: [mach smallestNumber > 0.0].! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testShowParameters
"included for coverage, test has no effect"
self should: [DhbFloatingPointMachine new showParameters isKindOf: DhbFloatingPointMachine ]! !

!DhbFloatingPointMachineTestCase methodsFor: 'precision' stamp: 'DanielUber 5/23/2012 14:16'!
testUniqueInstance
| mach1 mach2 mach3 |
mach1 := DhbFloatingPointMachine new.
mach2 := DhbFloatingPointMachine new.
self should: [mach1 == mach2].
DhbFloatingPointMachine reset.
mach3 := DhbFloatingPointMachine new.
self shouldnt: [mach3 == mach2].
! !

TestCase subclass: #DhbGeneticOptimizerBugsTest
	instanceVariableNames: 'manager function optimizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbGeneticOptimizerBugsTest methodsFor: 'running' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
setUp
manager:= DhbChromosomeManager new.
manager populationSize:3.
function:=[:x| (x*x)sum].
optimizer:= DhbGeneticOptimizer minimizingFunction: function.
optimizer chromosomeManager: manager.! !

!DhbGeneticOptimizerBugsTest methodsFor: 'tests' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testBug1
| r |
optimizer addPointAt: #(2 3).
optimizer addPointAt: #(2 3).
self shouldnt: [r:=optimizer randomScale] raise: Error.
self assert: ( r first equalsTo: (1/2) ).
self assert: ( r last equalsTo: 1).! !

!DhbGeneticOptimizerBugsTest methodsFor: 'tests' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testBug2
| r |
optimizer addPointAt: #(2 3).
"i add a nan by hand here,which in a way doesnt make sense, just to get a nan result for function, but with a more complicated function it is perfectly possible for the optimizer to produce nan's by itself!!"
optimizer addPointAt: {Float nan. 3}.
"if you inspect r here you will see that a _single nan result can _completely throw off the ga calculations"
r:=optimizer randomScale.
self deny: r first isNaN.! !

TestCase subclass: #DhbHistogramTestsAndBugs
	instanceVariableNames: 'h'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbHistogramTestsAndBugs methodsFor: 'comment' stamp: 'WernerKassens 4/11/2014 13:53'!
readMe
"DhbHistogram is insofar a bit problematic as for most methods it first needs the cache to be flushed by hand, but DhbHistogram is used by some other dhb objects that tend to forget to do this. hence i had to add some minimum amount of flushCache routines, so that everything works properly which is tested here (and things didnt slowdown, i made some routines a tiny bit faster). since the cache gets flushed automatically when the cache is full and the default cachesize is 100 and with 100 datapoints there is no noticable speed difference with using a cache, i would guess that the author had set the default size so low that the routines would work in most cases (but - well - just in most cases). anyway i did not change the default cache size. a real bug was corrected and tested in testCountsBetween and also in testDhbHistogrammedDistribution ."! !

!DhbHistogramTestsAndBugs methodsFor: 'running' stamp: 'WernerKassens 4/3/2014 21:22'!
setUp
h:=DhbHistogram new.
h freeExtent: true.
1to:3 do: [:i |h accumulate: i].! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 22:20'!
testAverage
self assert: (h average= 2).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/6/2014 13:15'!
testBinWidth
self assert: (h binWidth = (1/25) ). ! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/6/2014 13:16'!
testChi2Against

self assert: ((h chi2Against: (DhbScaledProbabilityDensityFunction histogram: h against:  DhbNormalDistribution new))
			closeTo:  2.937623). ! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 23:54'!
testCount
self assert: (h count= 3).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/4/2014 00:52'!
testCountAt
self assert: ((h countAt:3) = 1).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/5/2014 15:26'!
testCountsBetween
self assert: ((h countsBetween: 2 and: 4)=2).
"here is a bug in the original method: h countsBetween: 2 and: 3 resulted in -50"
self assert: ((h countsBetween: 2 and: 3)=1).
self assert: ((h countsBetween: 1 and: 3)=2).
self assert: ((h countsBetween: 2 and: 4)=2).
self assert: ((h countsBetween: 1 and: 4)=3).
self assert: ((h countsBetween: 2.1 and: 2.2)=0).
self assert: ((h countsBetween: 2 and: 2.04)=1).			"binWidth is 0.04"
self assert: ((h countsBetween: 2 and: 2.01)closeTo:0.25).	"hence this the correct result"
self assert: ((h countsBetween: 2.01 and: 3)closeTo: 0.75)."and so on..."
self assert: ((h countsBetween: 1.01 and: 2.01)closeTo:1).
self assert: ((h countsBetween: 2.01 and: 2.02)closeTo:0.25).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/5/2014 19:22'!
testCountsUpTo
self assert: ((h countsUpTo:  4) =3).
self assert: ((h countsUpTo:  2.1) =2).
self assert: ((h countsUpTo:  3) =2). 
self assert: ((h countsUpTo:  2) =1).
self assert: ((h countsUpTo:  1.04) =1).			"at 1.04 a new bin starts"
self assert:((h countsUpTo: 1.02)closeTo: 0.5).	"hence this is correct"
self assert: ((h countsUpTo:  1) =0).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/9/2014 14:27'!
testDhbHistogrammedDistribution
| g|
g:=DhbHistogrammedDistribution histogram: h.
"this raised an error (and printOn: did not work) because #parameters was not defined:"
self shouldnt: [ g printString ] raise: Error.
! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 23:24'!
testErrorOnAverage
self assert: (h errorOnAverage isFloat).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 23:31'!
testFConfidenceLevel
self assert: ((h fConfidenceLevel:h) = 100.0).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/6/2014 12:59'!
testInverseDistributionValue
self assert: ((h inverseDistributionValue: 1) = h maximum).
self assert: ((h inverseDistributionValue: 0) = h minimum).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/6/2014 10:29'!
testKurtosis
h accumulate: 2.
self assert: ( h kurtosis = (3/2) ).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/4/2014 00:53'!
testLowBinLimitAt
self assert: ((h lowBinLimitAt:2) isNumber).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/5/2014 13:15'!
testMaximum
self assert: ( h maximum >=3).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/5/2014 13:09'!
testMaximumCount
self assert: ( h maximumCount= 1 ).
h accumulate: 3.
self assert: ( h maximumCount= 2 ).
! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/5/2014 13:15'!
testMinimum
self assert: ( h minimum <=  1).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/4/2014 00:16'!
testSkewness
self assert: ( h skewness =0  ).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 22:50'!
testStandardDeviation
self assert: ( h standardDeviation =1 ).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 23:46'!
testTConfidenceLevel
self assert: ((h tConfidenceLevel:h) = 0).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 22:53'!
testTotalCount
self assert: (h totalCount= 3).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 22:54'!
testUnnormalizedVariance
self assert: (h unnormalizedVariance =2).! !

!DhbHistogramTestsAndBugs methodsFor: 'tests' stamp: 'WernerKassens 4/3/2014 22:55'!
testVariance
self assert: (h variance =1).! !

TestCase subclass: #DhbMatrixBug
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbMatrixBug methodsFor: 'tests' stamp: 'WernerKassens 4/13/2014 11:44'!
testAccumulate
"|m s|
m :=DhbMatrix rows: #((1 2 3)(-1 -2 7)(5 6 7)).
s :=DhbSymmetricMatrix rows: #((1 2 3)(2 1 4)(3 4 0)).
s accumulate: m."
"now s is not symmetric anymore, hence this should work, but it does not:"
"self deny: s isSymmetric ."
"the same thing is true for accumulateNegated: .
(a accumulate: b) is essentially the same as (a:=a+b) .
both methods are used in the symmetric case only (!!) in methods crlInverse and inversePureCRL. and they are not used to accumulate several matrices. obvously the only reason for their existence is that they are somewhat faster than a:=a+b and a:=a-b. but now, because the addition of symmetric matrices is optimized, there is no problem in using normal addition. hence i removed them "
! !

!DhbMatrixBug methodsFor: 'tests' stamp: 'WernerKassens 4/14/2014 12:56'!
testSimpleMatrixOperations
|s m s2 r r2|
"a mix of unsymmetric and symmetric matrices"
m :=DhbMatrix rows: #((1 2 3)(-1 -2 7)(5 6 7)).
s :=DhbSymmetricMatrix rows: #((1 2 3)(2 1 4)(3 4 0)).
s2 :=DhbSymmetricMatrix rows: #((1 2 0)(2 1 4)(0 4 0)).
"just to be sure that s and s2 are indeed symmetric:"
self assert: (s=s transpose ).
self assert: (s2=s2 transpose ).
r:=m*s.
self assert: (r =(DhbMatrix rows: #((14 16 11)(16 24 -11)(38 44 39)))).
self assert: (r class=DhbMatrix).
r:=s*m.
self assert: (r =(DhbMatrix rows: #((14 16 38)(21 26 41)(-1 -2 37)))).
self assert: (r class=DhbMatrix).
r:=s*s2.
self deny: (r=r transpose ).	"hence this (here is the bug) must be true:"
self assert: (r class=DhbMatrix).
self assert: (r =(DhbMatrix rows: #((5 16 8)(4 21 4)(11 10 16)))).
r:=s transposeProductWithMatrix:s2.
self deny: (r=r transpose ).	"hence this must be true:"
self assert: (r class=DhbMatrix).
r:=s productWithTransposeMatrix:s2.
self deny: (r=r transpose ).	"hence this must be true:"
self assert: (r class=DhbMatrix).

self assert: (s*s=s squared).
self assert: ((m transpose * m )=m squared).
self assert: ((r:=m+s)=(r2:=s+m)).
self assert: (r class=DhbMatrix).
self assert: (r2 class=DhbMatrix).
r2:=DhbMatrix rows: #((0 0 0)(-3 -3 3)(2 2 7)).
self assert: ((r:=m-s)=r2).
self assert: (r class=DhbMatrix).
r:=(s-m) negate .
self assert: (r=r2).
self assert: (r class=DhbMatrix).
self assert: ((r:=s2+s)=(s+s2)).
self assert: (r class=DhbSymmetricMatrix).
self assert: ((r:=s2-s)=(s-s2) negate) .
self assert: (r class=DhbSymmetricMatrix).
m:=DhbMatrix rows: #((-1 -2 7)(5 6 7)).
self assert: (m*s=(s*m transpose )transpose).
self assert: ((m transpose * m )=m squared).
! !

!DhbMatrixBug methodsFor: 'tests' stamp: 'WernerKassens 5/2/2014 14:35'!
testSkalarMultiplication
|a r|
a:=DhbMatrix rows: #((1 2)(3 3)).
"this was intended by the author to be possible; it can be seen by the code and the book:"
r:=a*2.
self assert:(r class=DhbMatrix).
self assert:(r=(DhbMatrix rows: #((2 4)(6 6)))).
self assert:(2*a=r).
a:=DhbSymmetricMatrix  rows: #((1 2)(2 1)).
r:=a*2.
self assert:(r=(DhbMatrix rows: #((2 4)(4 2)))).
self assert: (r class=DhbSymmetricMatrix).
r:=2.0*a.
self assert:(r=(DhbMatrix rows: #((2.0 4.0)(4.0 2.0)))).
self assert: (r class=DhbSymmetricMatrix).
! !

!DhbMatrixBug methodsFor: 'tests' stamp: 'WernerKassens 4/15/2014 09:56'!
testSymmetric
|a m|
a:=#(1 2 3)asDhbVector.
m:=DhbMatrix rows: #((1 2)(3 3)).
self deny: (m isSymmetric ).
"but a DhbMatrix can of course also be symmetric, 
and it is occasionally interesting to know, 
hence the result should be correct"
m:=DhbMatrix rows: (a tensorProduct: a)rows.
self assert: (m isSymmetric ).

! !

TestCase subclass: #DhbNumberExtensionsTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbNumberExtensionsTestCase commentStamp: '<historical>' prior: 0!
A DhbNumberExtensionsTestCase is a suite of tests for extension methods to Number, Integer, and Float defined in Didier Besset's book Object Oriented Numerical Methods!

!DhbNumberExtensionsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:18'!
testBeta
	"Code example 2.14"
	| value |

	value := 2.5 gamma * 5.5 gamma / 8 gamma.
	self should: [ ((2.5 beta: 5.5)  - value) abs < 1.0e-14].! !

!DhbNumberExtensionsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:18'!
testBetaLog
	"Code example 2.15"
	| value |
	value := ( 2.5 gamma * 5.5 gamma / 8 gamma) ln.
	self should: [ ((2.5 logBeta: 5.5)  - value) abs < 1.0e-13].! !

!DhbNumberExtensionsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:19'!
testErrorFunctionCentile
	"Code example 2.5"
	| weight average stDev centile |
	weight := 2.85.
	average := 3.39.
	stDev := 0.44.
	centile := (( weight - average) / stDev) errorFunction * 100.
	self should: [ ( centile - 10.986012) abs < 0.000001].! !

!DhbNumberExtensionsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:20'!
testGamma
	"Code example 2.10"
	| value |
	value := Float pi sqrt * 3 / 4.
	self should: [ (2.5 gamma  - value) abs < 1.0e-14].! !

!DhbNumberExtensionsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:20'!
testGammaLog
	"Code example 2.11"
	| value |
	value := 2.5 gamma ln.
	self should: [ (2.5 logGamma  - value) abs < 1.0e-13].! !

!DhbNumberExtensionsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:20'!
testGammaLow

	| value |
	value := Float pi sqrt / 2.
	self should: [ ((3/2) gamma  - value) abs < 1.0e-14].! !

!DhbNumberExtensionsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:20'!
testGammaNegative

	| value |
	value := Float pi / ( 1.5 gamma * (Float pi / -2) sin).
	self should: [ ((-1/2) gamma  - value) abs < 1.0e-14].! !

TestCase subclass: #DhbNumericalMethodsTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbNumericalMethodsTestCase methodsFor: 'privateMethods' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
accumulateAround: aVector size: aNumber into: aCollection
	"Private - Generate a random point around the given center and insert it into the collection.
					aNumber is the sigma for the distance to the center"
	| r phi psi localVector |

	r := ( DhbNormalDistribution new: 0 sigma: aNumber) random.
	phi := Float pi random.
	psi := Float pi random.
	localVector := DhbVector new: 3.
	localVector at: 1 put: ( phi sin * psi sin * r );
					   at: 2 put: ( phi cos * psi sin * r );
					   at: 3 put: ( psi cos * r ).
	aCollection add: (localVector + aVector).! !

!DhbNumericalMethodsTestCase methodsFor: 'privateMethods' stamp: 'WernerKassens 4/15/2014 09:57'!
generatedPoints: anInteger
	"Private - Generate random points into aCollection. 3 clusters are used"
	| centers results |
	centers := Array new: 3.
	centers at: 1 put: #( 200 200  200) asDhbVector;
				 at: 2 put: #(-200 200  200) asDhbVector;
				 at: 3 put: #( 200 200 -200) asDhbVector.
	results := OrderedCollection new.
	anInteger timesRepeat: [ self accumulateAround: ( centers at: (3 random + 1)) size: 1 into: results].
	^results! !

!DhbNumericalMethodsTestCase methodsFor: 'privateMethods' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
setUp
	"Reset the seed of the random numbers (to get consistent results)"
	DhbMitchellMooreGenerator reset: 0.
	"Reset the FloatingPointMachine to make coverage consistent"
	DhbFloatingPointMachine reset.! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 5/2/2014 18:35'!
testAtRowPutAtColumnPut
	| a  |
	a := DhbMatrix rows: #( ( 11 12 13) (21 22 23)).
	a atRow: 1 put: (a rowAt: 2).
	self assert: a=(DhbMatrix rows: #( (21 22 23) (21 22 23))).
	a atColumn: 3 put: (a columnAt: 2).
	self assert: a=(DhbMatrix rows: #( (21 22 22) (21 22 22))).
	a := DhbSymmetricMatrix rows: #( ( 11 12 ) (21 22)).
	self should: [ a atRow: 1 put: (a rowAt: 2) ] raise: Error.
	self should: [ a atColumn: 1 put: (a rowAt: 2) ] raise: Error
! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testBissection
	"Code Example 5.1"
	| zeroFinder result |
	zeroFinder := DhbBisectionZeroFinder function: [ :x | x errorFunction - 0.9].
	zeroFinder setPositiveX: 10.0; setNegativeX: 0.0.
	result := zeroFinder evaluate.
	self should: [ zeroFinder hasConverged].
	self should: [ (result -1.28155193291605) abs < 1.0e-14]! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testClusterCovariance

	| dataServer clusters finder |

	dataServer := DhbMemoryBasedDataServer new.
	dataServer data: ( self generatedPoints: 1000).
	finder := DhbClusterFinder new: 5 server: dataServer type: DhbCovarianceCluster.
	finder minimumRelativeClusterSize: 0.1.
	clusters := finder evaluate.
	self should: [ clusters size = 3].! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testClusterEuclidean

	| dataServer clusters finder |
	dataServer := DhbMemoryBasedDataServer new.
	dataServer data: ( self generatedPoints: 1000).
	finder := DhbClusterFinder new: 5 server: dataServer type: DhbEuclideanCluster.
	finder minimumRelativeClusterSize: 0.15.
	clusters := finder evaluate.
	self should: [ clusters size = 3].! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'WernerKassens 4/15/2014 09:56'!
testCovarianceAccumulation
	"Code example 12.2"
	| accumulator average covarianceMatrix |
	accumulator := DhbCovarianceAccumulator new: 3.
	#( 
			(1 2 3)
			( 2 3 4)
			( 1 3 2)
			( 4 3 1)
			( 1 3 1)
			( 1 4 2)
			( 3 1 2)
			( 3 4 2)
		)
	do: [ :x | accumulator accumulate: x asDhbVector].
	average := accumulator average.
	self should: [ ( average at: 1) equalsTo: 2.0].
	self should: [ ( average at: 2) equalsTo: 2.875].
	self should: [ ( average at: 3) equalsTo: 2.125].
	covarianceMatrix := accumulator covarianceMatrix.
	self should: [ ((covarianceMatrix rowAt: 1) at: 1) equalsTo: 1.25].
	self should: [ ((covarianceMatrix rowAt: 1) at: 2) equalsTo: -0.125].
	self should: [ ((covarianceMatrix rowAt: 2) at: 1) equalsTo: -0.125].
	self should: [ ((covarianceMatrix rowAt: 1) at: 3) equalsTo: -0.25].
	self should: [ ((covarianceMatrix rowAt: 3) at: 1) equalsTo: -0.25].
	self should: [ ((covarianceMatrix rowAt: 2) at: 2) equalsTo: 0.859375].
	self should: [ ((covarianceMatrix rowAt: 2) at: 3) equalsTo: -0.109375].
	self should: [ ((covarianceMatrix rowAt: 3) at: 2) equalsTo: -0.109375].
	self should: [ ((covarianceMatrix rowAt: 3) at: 3) equalsTo: 0.859375].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testDeterminant

	| m |
	m := DhbMatrix rows: #( (3 2 4) (2 -5 -1) ( 1 -2 2)).
	self should: [ m determinant = -42].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testEigenvalues
	"Code Example 8.15"
	| m charPol roots eigenvalues finder |
	m := DhbMatrix rows: #( ( 3 -2 0) (-2 7 1) (0 1 5)).
	charPol := DhbPolynomial coefficients: #(82 -66 15 -1).
	roots := charPol roots asSortedCollection asArray reverse.
	finder := DhbJacobiTransformation matrix: m.
	finder desiredPrecision: 1.0e-09.
	eigenvalues := finder evaluate.
	self should: [ eigenvalues size = 3].
	self should: [ ((roots at: 1) - (eigenvalues at: 1)) abs < 1.0e-09].
	self should: [ ((roots at: 2) - (eigenvalues at: 2)) abs < 1.0e-09].
	self should: [ ((roots at: 3) - (eigenvalues at: 3)) abs < 1.0e-09].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testEigenvaluesLargest
	"Code Example 8.13"
	| m charPol roots eigenvalue finder |
	m := DhbMatrix rows: #( ( 3 -2 0) (-2 7 1) (0 1 5)).
	charPol := DhbPolynomial coefficients: #(82 -66 15 -1).
	roots := charPol roots asSortedCollection asArray reverse.
	finder := DhbLargestEigenValueFinder matrix: m.
	finder desiredPrecision: 1.0e-08.
	eigenvalue := finder evaluate.
	self should: [ ((roots at: 1) - eigenvalue) abs < 1.0e-08].
	finder := finder nextLargestEigenValueFinder.
	eigenvalue := finder evaluate.
	self should: [ ((roots at: 2) - eigenvalue) abs < 1.0e-08].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testErrorFunction
	"simple cases to expect"
	self should: [0 errorFunction = (1 / 2)].
	self should:
		[ DhbFloatingPointMachine new  largestNumber errorFunction > 
 		(1 - DhbFloatingPointMachine new machinePrecision  )].
	"add some code to require initialize to run"
	DhbErfApproximation reset.
	self should:
		[DhbFloatingPointMachine new largestNumber negated errorFunction < DhbFloatingPointMachine new smallestNumber ].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testFTest

	| accC accMM confidenceLevel|
	accC := DhbStatisticalMoments new.
	#( 5.56 5.89 4.66 5.69 5.34 4.79 4.80 7.86 3.64 5.70 )
		do: [ :x | accC accumulate: x].
	accMM := DhbStatisticalMoments new.
	#( 7.48 6.75 3.77 5.71 7.25 4.73 6.23 5.60 5.94 4.58 )
		do: [ :x | accMM accumulate: x].
	confidenceLevel := accC fConfidenceLevel: accMM.
	self should: [ (accC average - 5.393) abs < 0.000000001].
	self should: [ (accC standardDeviation - 1.0990809292) abs < 0.000000001].
	self should: [ (accMM average - 5.804) abs < 0.000000001].
	self should: [ (accMM standardDeviation - 1.19415428) abs < 0.000000001].
	self should: [ (confidenceLevel - 79.8147614536) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testGammaDistribution
	| dist |
	dist := DhbGammaDistribution shape: 3.4 scale: 1.7.
	self should: [ (dist average - (3.4 * 1.7)) abs < 0.000000001].
	self should: [ (dist standardDeviation - (3.4 sqrt *1.7)) abs < 0.000000001].
	self should: [ ((dist value: 4.5) - 0.1446067652) abs < 0.000000001].
	self should: [ ((dist distributionValue: 4.5) - 0.3982869736) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testHistogram
	| histogram |
	histogram := DhbHistogram new.
	histogram setRangeFrom: 0.0 to: 48.0 bins: 8.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | histogram accumulate: x].
	histogram accumulate: -1; accumulate: 55; accumulate: 56.
	self should: [ histogram count = 30].
	self should: [ histogram underflow = 1].
	self should: [ histogram overflow = 2].
	self should: [ ( histogram countAt: 1) = 3].
	self should: [ ( histogram countAt: 8.5) = 4].
	self should: [ ( histogram countAt: 16) = 8].
	self should: [ ( histogram countAt: 23.5) = 4].
	self should: [ ( histogram countAt: 31) = 6].
	self should: [ ( histogram countAt: 38.5) = 4].
	self should: [ ( histogram countAt: 46) = 1].
	self should: [ (histogram average - 24.1333333333) abs < 0.000000001].
	self should: [ (histogram standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [ (histogram skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (histogram kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testIncompleteBetaFunction

	| function |
	function := DhbIncompleteBetaFunction shape: 2 shape: 5.
	self should: [ ( (function value: 0.8) - 0.9984) abs < 0.00001].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testIncompleteGammaFunction

	| function |
	function := DhbIncompleteGammaFunction shape: 2.
	self should: [ ( (function value: 2) - 0.59399414981) abs < 0.00000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testIntegrationRomberg

	| integrator ln2 ln3|
	integrator := DhbRombergIntegrator function: [ :x | 1.0 / x] from: 1 to: 2.
	ln2 := integrator evaluate.
	integrator from: 1 to: 3.
	ln3 := integrator evaluate.
	self should: [ ( 2.0 ln - ln2) abs < ( 2 * integrator precision)].
	self should: [ ( 3.0 ln - ln3) abs < ( 2 * integrator precision)].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testIntegrationSimpson

	| integrator ln2 ln3|
	integrator := DhbSimpsonIntegrator function: [ :x | 1.0 / x] from: 1 to: 2.
	ln2 := integrator evaluate.
	integrator from: 1 to: 3.
	ln3 := integrator evaluate.
	self should: [ ( 2.0 ln - ln2) abs < integrator precision].
	self should: [ ( 3.0 ln - ln3) abs < integrator precision].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testIntegrationTrapeze
	"Code Example 6.1"
	| integrator ln2 ln3|
	integrator := DhbTrapezeIntegrator function: [ :x | 1.0 / x] from: 1 to: 2.
	ln2 := integrator evaluate.
	integrator from: 1 to: 3.
	ln3 := integrator evaluate.
	self should: [ ( 2.0 ln - ln2) abs < integrator precision].
	self should: [ ( 3.0 ln - ln3) abs < integrator precision].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationBulirschStoer

	| interpolator |
	interpolator := DhbBulirschStoerInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationLagrange
	"Code example 3.2"
	| interpolator |
	interpolator := DhbLagrangeInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationLagrangeLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbLagrangeInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationNeville

	| interpolator |

	interpolator := DhbNevilleInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationNevilleLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbNevilleInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationNewton

	| interpolator |
	interpolator := DhbNewtonInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationNewtonLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbNewtonInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationSpline

	| interpolator |
	interpolator := DhbSplineInterpolator new.
	1 to: 45 by: 2 do:
		[ :x | interpolator add: x @ (x degreesToRadians sin)].
	self should: [ ((interpolator value: 8) - (8 degreesToRadians sin)) abs < 1.0e-7].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testInterpolationSplineLinear
	"Code example 3.1"
	| interpolator |
	interpolator := DhbSplineInterpolator points: (Array with: 1 @ 2 with: 3 @ 1 ).
	self should: [ ((interpolator value: 2.2) - 1.4) abs < 1.0e-14].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLUPDecomposition
	"Code Example 8.10"
	| s sol1 sol2 |
	s := DhbLUPDecomposition equations:
							#( (3 2 4) (2 -5 -1) ( 1 -2 2)).
	sol1 := s solve: #(16 6 10).
	sol2 := s solve: #(7 10 9).
	self should: [ sol1 size = 3].
	self should: [ (sol1 at: 1) = 2].
	self should: [ (sol1 at: 2) = -1].
	self should: [ (sol1 at: 3) = 3].
	self should: [ sol2 size = 3].
	self should: [ (sol2 at: 1) = 1].
	self should: [ (sol2 at: 2) = -2].
	self should: [ (sol2 at: 3) = 2].! !

!DhbNumericalMethodsTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLanczosFormulaObject
	"verify that initialize is sent at least once per test run"
	| first second third |
	first := DhbLanczosFormula new.
	DhbLanczosFormula reset.
	second := DhbLanczosFormula new.
	self shouldnt: [first == second].
	third := DhbLanczosFormula new.
	self should: [second == third].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLeastSquare
	"Code example 10.9"
	"Note: the seemingly large error on the fit results is due to the binning of the histogram."
	| count shape scale genDistr hist fit fittedDistr parameters |
	count := 10000.
	shape := 0.
	scale := 1.
	hist := DhbHistogram new.
	hist freeExtent: true.
	genDistr := DhbFisherTippettDistribution shape: shape scale: scale.
	count timesRepeat: [ hist accumulate: genDistr random].
	fit := DhbLeastSquareFit histogram: hist distributionClass: DhbFisherTippettDistribution.
	fittedDistr := fit evaluate.
	parameters := fittedDistr parameters.
	self should: [ ((parameters at: 1) - shape) abs < 0.1].
	self should: [ ((parameters at: 2) - scale) abs < 0.1].
	self should: [ ((parameters at: 3) - count) abs < 100].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLeastSquarePolynomial
	"Code example 10.5"
	| fit estimation |
	fit := DhbPolynomialLeastSquareFit new: 3.
	fit
		add: (DhbWeightedPoint point: 1 @ 2.0);
		add: (DhbWeightedPoint point: 2 @ 21.0);
		add: (DhbWeightedPoint point: 3 @ 72.0);
		add: (DhbWeightedPoint point: 4 @ 173.0);
		add: (DhbWeightedPoint point: 5 @ 342.0);
		add: (DhbWeightedPoint point: 6 @ 597.0);
		add: (DhbWeightedPoint point: 7 @ 956.0);
		add: (DhbWeightedPoint point: 8 @ 1437.0);
		add: (DhbWeightedPoint point: 9 @ 2058.0);
		add: (DhbWeightedPoint point: 10 @ 2837.0).
	estimation := fit evaluate.
	self should: [ ((estimation value: 4.5)- 247.875) abs < 0.000000001].
	self should: [ ((estimation error: 4.5) -  5.215298e-1) abs < 0.00001].
	self should: [ ((estimation value: 7.15)- 1019.932625) abs < (estimation error: 7.15)].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLinearEquations
	"Code Example 8.6"
	| s sol1 sol2 |
	s := DhbLinearEquationSystem equations:
							#( (3 2 4) (2 -5 -1) ( 1 -2 2))
				constants: #( (16 6 10)
									  (7 10 9)).
	sol1 := s solutionAt: 1.
	sol2 := s solutionAt: 2.
	self should: [ sol1 size = 3].
	self should: [ (sol1 at: 1) = 2].
	self should: [ (sol1 at: 2) = -1].
	self should: [ (sol1 at: 3) = 3].
	self should: [ sol2 size = 3].
	self should: [ (sol2 at: 1) = 1].
	self should: [ (sol2 at: 2) = -2].
	self should: [ (sol2 at: 3) = 2].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLinearEquationsSingle

	| s sol |
	s := DhbLinearEquationSystem equations:
							#( (1 2 0) (3 5 4) (5 6 3)) constant: #(0.1 12.5 10.3).
	sol := s solution.
	self should: [ sol size = 3].
	self should: [ (sol at: 1) equalsTo: 0.5].
	self should: [ (sol at: 2) equalsTo: -0.2].
	self should: [ (sol at: 3) equalsTo:  3.0].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLinearEquationsSingular

	| s sol |
	s := DhbLinearEquationSystem equations:
							#( (1 2 0) (10 12 6) (5 6 3)) constant: #(0.1 12.5 10.3).
	sol := s solution.
	self should: [ sol isNil].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testLinearRegression
	"Code example 10.5"
	| linReg estimation |
	linReg := DhbLinearRegression new.
	linReg
		add: 1 @ 0.72;
		add: 2 @ 3.25;
		add: 3 @ 5.75;
		add: 4 @ 8.21;
		add: 5 @ 10.71;
		add: 6 @ 13.38;
		add: 7 @ 15.82;
		add: 8 @ 18.39;
		add: 9 @ 20.72;
		add: 10 @ 23.38.
	self should: [ (linReg slope - 2.514727272727) abs < 0.000000000001].
	self should: [ (linReg intercept + 1.798) abs < 0.000000000001].
	self should: [ (linReg correlationCoefficient - 0.999966922113) abs < 0.000000000001].
	estimation := linReg asEstimatedPolynomial.
	self should: [ ((estimation value: 4.5)- 9.5182727272727) abs < 0.000000000001].
	self should: [ ((estimation value: 7.15)-  16.1823) abs < 0.000000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'data mining' stamp: 'WernerKassens 4/15/2014 09:57'!
testMahalanobisCenter
	"Code example 12.5"
	| center distance|
	center := DhbMahalanobisCenter new: 3.
	#( 
			(1 2 3)
			( 2 3 4)
			( 1 3 2)
			( 4 3 1)
			( 1 3 1)
			( 1 4 2)
			( 3 1 2)
			( 3 4 2)
		)
	do: [ :x | center accumulate: x asDhbVector].
	center computeParameters.
	distance := center distanceTo: #(1 2 3) asDhbVector.
	self should: [ distance equalsTo: 2.26602282704126].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMatrixAdd
	| a b c |
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	b := DhbMatrix rows: #( ( 1 2 3) (-2 1 7)).
	c := a + b.
	self should: [ c numberOfRows = 2].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 2].
	self should: [ ((c rowAt: 1) at: 2) = 2].
	self should: [ ((c rowAt: 1) at: 3) = 4].
	self should: [ ((c rowAt: 2) at: 1) = -3].
	self should: [ ((c rowAt: 2) at: 2) = -1].
	self should: [ ((c rowAt: 2) at: 3) = 10].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 5/2/2014 15:52'!
testMatrixDo
	| a |
	a := DhbMatrix rows: #((1 2 3) ( 1 2 3) (1 2 3)).
	a rowsDo: [:row | row at: 1 put: 0].
	self should: [(a rowAt: 1 columnAt: 1) = 0].
	a columnsDo: [:col | a atRow: 1 put: col].
	self should: [(a rowAt: 1) = (#(2 3 3) asDhbVector)].
! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMatrixEquality
	| a c |
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	c := a .
	self should: [ c numberOfRows =  a numberOfRows ].
	self should: [ c numberOfColumns = a numberOfColumns ].
	self should: [c = a and: [ a = c]].
! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 7/22/2014 13:27'!
testMatrixExtensions
	"testing at:at: and at:at:put:"
	| a c |
	(RPackage  organizer includesPackageNamed:'Math-DHB-NumericalExtensions') ifFalse:[^self skip].
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	c := a deepCopy .
	self should: [(c at: 1 at: 1) = (a at: 1 at: 1)].
	a at: 1 at: 1 put: 42.
	self shouldnt: [(c at: 1 at: 1 )= (a at: 1 at: 1)].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 7/22/2014 13:27'!
testMatrixExtensionsAtColumn
	"testing at:at: and at:at:put:"
	| a c |
	(RPackage  organizer includesPackageNamed:'Math-DHB-NumericalExtensions') ifFalse:[^self skip].
	a := DhbMatrix rows: #( ( 11 12 13) (21 22 23)).
	c := a deepCopy .
	self should: [(c at: 1 at: 1) = (a at: 1 at: 1)].
	c atColumn: 1 put: (a atColumn: 2).
	self shouldnt: [(c at: 1 at: 1 )= (a at: 1 at: 1)].
	self should: [(c at: 1 at: 1) = (a at: 1 at: 2)].
	c := a deepCopy .
	c at: 1at: 1 put: (a at: 1 at: 2). 
	c atColumn: 1 put: (a atColumn: 2) startingAt: 1. 

	self should: [(c at: 2 at: 1 ) = (a at: 1 at: 2) ].
	self shouldnt: [(c at: 1 at: 1) = (a at: 1 at: 1)].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 7/22/2014 13:28'!
testMatrixExtensionsAtRow
	"testing at:at: and at:at:put:"
	| a c |
	(RPackage  organizer includesPackageNamed:'Math-DHB-NumericalExtensions') ifFalse:[^self skip].
	a := DhbMatrix rows: #( ( 11 12 13) (21 22 23)).
	c := a deepCopy .
	self should: [(c at: 1 at: 1) = (a at: 1 at: 1)].
	c atRow: 1 put: (a rowAt: 2).
	self shouldnt: [(c at: 1 at: 1 )= (a at: 1 at: 1)].
	self should: [(c at: 1 at: 1) = (a at: 2 at: 1)].
	c := a deepCopy .
	c atRow: 1 put: (a rowAt: 2) startingAt: 1. 

	self should: [(c at: 1 at: 2 ) = (a at: 2 at: 1) ].
	self shouldnt: [(c at: 1 at: 2) = (a at: 1 at: 2)].! !

!DhbNumericalMethodsTestCase methodsFor: 'comparing' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMatrixHash
	| a b c |
	a := DhbMatrix rows: #( (1 0) (0 1)).
	b := a deepCopy.
	self should: [a hash = b hash].
	c := a + b.
	self shouldnt: [a hash = c hash].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMatrixInitialize
	| a |
	a := DhbMatrix new initialize: 2.
	self should: [a numberOfRows = 2].
	self should: [a numberOfColumns = 2].
	! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMatrixInversionSmall

	| m c |
	m := DhbMatrix rows: #( (3 2 4) (2 -5 -1) ( 1 -2 2)).
	c := m inverse * m.
	self should: [ c numberOfRows = 3].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 1].
	self should: [ ((c rowAt: 1) at: 2) = 0].
	self should: [ ((c rowAt: 1) at: 3) = 0].
	self should: [ ((c rowAt: 2) at: 1) = 0].
	self should: [ ((c rowAt: 2) at: 2) = 1].
	self should: [ ((c rowAt: 2) at: 3) = 0].
	self should: [ ((c rowAt: 3) at: 1) = 0].
	self should: [ ((c rowAt: 3) at: 2) = 0].
	self should: [ ((c rowAt: 3) at: 3) = 1].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMatrixMultiply
	"Code Example 8.1"
	| a b c |
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	b := DhbMatrix rows: #( ( 1 2 3) (-2 1 7) (5 6 7)).
	c := a * b.
	self should: [ c numberOfRows = 2].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 6].
	self should: [ ((c rowAt: 1) at: 2) = 8].
	self should: [ ((c rowAt: 1) at: 3) = 10].
	self should: [ ((c rowAt: 2) at: 1) = 18].
	self should: [ ((c rowAt: 2) at: 2) = 14].
	self should: [ ((c rowAt: 2) at: 3) = 4].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMatrixSubtract
	| a b c |
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	b := DhbMatrix rows: #( ( 1 2 3) (-2 1 7)).
	c := a - b.
	self should: [ c numberOfRows = 2].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 0].
	self should: [ ((c rowAt: 1) at: 2) = -2].
	self should: [ ((c rowAt: 1) at: 3) = -2].
	self should: [ ((c rowAt: 2) at: 1) = 1].
	self should: [ ((c rowAt: 2) at: 2) = -3].
	self should: [ ((c rowAt: 2) at: 3) = -4].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testMaximumLikelihood
	"Code example 10.11"
	"Note: the seemingly large error on the fit results is due to the binning of the histogram."
	| count shape scale genDistr hist fit fittedDistr parameters |
	count := 10000.
	shape := 0.
	scale := 1.
	hist := DhbHistogram new.
	hist freeExtent: true.
	genDistr := DhbFisherTippettDistribution shape: shape scale: scale.
	count timesRepeat: [ hist accumulate: genDistr random].
	fit := DhbMaximumLikekihoodHistogramFit histogram: hist distributionClass: DhbFisherTippettDistribution.
	fittedDistr := fit evaluate.
	parameters := fittedDistr parameters.
	self should: [ ((parameters at: 1) - shape) abs < 0.1].
	self should: [ ((parameters at: 2) - scale) abs < 0.1].
	self should: [ ((parameters at: 3) - count) abs < 100].! !

!DhbNumericalMethodsTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testNewtonZeroFinder
	"Code Example 5.3"
	| zeroFinder result |
	zeroFinder := DhbNewtonZeroFinder function: [ :x | x errorFunction - 0.9].
	zeroFinder initialValue: 1.0.
	result := zeroFinder evaluate.
	self should: [ zeroFinder hasConverged].
	self should: [ (result -1.28155193867885) abs < zeroFinder precision]! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testNormalDistribution
	| dist |
	dist := DhbNormalDistribution new: 3.4 sigma: 1.7.
	self should: [ (dist average - 3.4) abs < 0.000000001].
	self should: [ (dist standardDeviation - 1.7) abs < 0.000000001].
	self should: [ ((dist value: 4.5) - 0.1903464693) abs < 0.000000001].
	self should: [ ((dist distributionValue: 4.5) - 0.7412031298) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'WernerKassens 4/15/2014 09:56'!
testOptimize
	"General optimizer to test genetic algorithm"
	| fBlock finder result |
	fBlock := [ :x | |r| r := x * x. r = 0 ifTrue: [ 1] ifFalse: [ r sqrt sin / r] ].
	finder := DhbMultiVariableGeneralOptimizer maximizingFunction: fBlock.
	finder desiredPrecision: 1.0e-6.
	finder origin: #( 0.5 1.0 0.5) asDhbVector; range: #(2 2 2) asDhbVector.
	result := finder evaluate.
	self should: [ finder precision < 1.0e-6].
	self should: [ (result at: 1) abs < 1.0e-6].
	self should: [ (result at: 2) abs < 1.0e-6].
	self should: [ (result at: 3) abs < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testOptimizeOneDimension
	"Code example 11.1"
	| distr finder maximum |
	distr := DhbGammaDistribution shape: 2 scale: 5.
	finder := DhbOneVariableFunctionOptimizer maximizingFunction: distr.
	finder desiredPrecision: 1.0e-6.
	maximum := finder evaluate.
	self should: [ ( maximum - 5) abs < 1.0e-6].
	self should: [ finder precision < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'WernerKassens 4/15/2014 09:56'!
testOptimizePowell
	"Code example 11.3"
	| fBlock hillClimber educatedGuess result |
	fBlock := [ :x | (x * x) negated exp].
	educatedGuess := #( 0.5 1.0 0.5) asDhbVector.
	hillClimber := DhbHillClimbingOptimizer maximizingFunction: fBlock.
	hillClimber initialValue: educatedGuess.
	hillClimber desiredPrecision: 1.0e-6.
	result := hillClimber evaluate.
	self should: [ hillClimber precision < 1.0e-6].
	self should: [ (result at: 1) abs < 1.0e-6].
	self should: [ (result at: 2) abs < 1.0e-6].
	self should: [ (result at: 3) abs < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'optimization' stamp: 'WernerKassens 4/15/2014 09:56'!
testOptimizeSimplex
	"Code example 11.5"
	| fBlock simplex educatedGuess result |
	fBlock := [ :x | (x * x) negated exp].
	educatedGuess := #( 0.5 1.0 0.5) asDhbVector.
	simplex := DhbSimplexOptimizer maximizingFunction: fBlock.
	simplex initialValue: educatedGuess.
	simplex desiredPrecision: 1.0e-6.
	result := simplex evaluate.
	self should: [ simplex precision < 1.0e-6].
	self should: [ (result at: 1) abs < 1.0e-6].
	self should: [ (result at: 2) abs < 1.0e-6].
	self should: [ (result at: 3) abs < 1.0e-6].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 5/2/2014 08:33'!
testSingularMatrixError
|h f|
h:=DhbHistogram new.
h freeExtent: true.
1 to: 3 do:  [:i| h accumulate: i ].
f:=DhbLeastSquareFit histogram: h distributionClass: DhbTriangularDistribution.
self should: [ f evaluate ] raise: SingularMatrixError .
"and not something completely incomprehensible"
"also here:"
self should: [ f errorMatrix ] raise: SingularMatrixError .
self should: [(DhbMatrix rows: #((1 2 3)(3 4 6)))inverse] raise: SingularMatrixError .
self should: [(DhbSymmetricMatrix rows: #((1 2 3)(2 2 2)(3 2 1)))inversePureLUP] raise: SingularMatrixError.
self should: [(DhbSymmetricMatrix rows: #((1 2 3)(2 2 2)(3 2 1)))inversePureCRL] raise: SingularMatrixError.! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testStatisticalMoments
	"comment"
	| accumulator |
	accumulator := DhbStatisticalMoments new.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | accumulator accumulate: x].
	self should: [ (accumulator average - 24.1333333333) abs < 0.000000001].
	self should: [ (accumulator standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [(accumulator skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (accumulator kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testStatisticalMomentsFast

	| accumulator |
	accumulator := DhbFastStatisticalMoments new.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | accumulator accumulate: x].
	self should: [ (accumulator average - 24.1333333333) abs < 0.000000001].
	self should: [ (accumulator standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [ (accumulator skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (accumulator kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'statistics' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testStatisticalMomentsFixed

	| accumulator |
	accumulator := DhbFixedStatisticalMoments new.
	#( 36 13 27 16 33 24 4 20 15 23 37 23 31 15 47 22  6 15 41 22  14 14  31 42 3 42 22 8  37 41 )
		do: [ :x | accumulator accumulate: x].
	self should: [ (accumulator average - 24.1333333333) abs < 0.000000001].
	self should: [ (accumulator standardDeviation - 12.461619237603) abs < 0.000000001].
	self should: [ (accumulator skewness - 0.116659884676) abs < 0.000000001].
	self should: [ (accumulator kurtosis + 1.004665562311) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testSymmetricMatrixAdd
	| a b c |
	a := (DhbMatrix rows: #( ( 11 12 13) (12 22 23) (13 23 33))) asSymmetricMatrix .
	b := DhbMatrix rows: #( ( 1 2 3) (-2 1 7) (0 0 0)).
	c := a + b.
	self should: [ c numberOfRows = 3].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 12].
	self should: [ ((c rowAt: 1) at: 2) = 14].
	self should: [ ((c rowAt: 1) at: 3) = 16].
! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testSymmetricMatrixAdd2
	| a b c |
	a := DhbSymmetricMatrix rows: #( ( 11 12 13) (12 22 23) (13 23 33)).
	b := DhbSymmetricMatrix rows: #( ( 1 2 3) (2 1 7) (3 7 0)).
	c := a + b.
	self should: [ c numberOfRows = 3].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 12].
	self should: [ ((c rowAt: 1) at: 2) = 14].
	self should: [ ((c rowAt: 1) at: 3) = 16].
! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testSymmetricMatrixAdd3
	| a b c |
	a := DhbMatrix rows: #( ( 11 12 13) (21 22 23) (31 32 33)).
	b := DhbSymmetricMatrix rows: #( ( 1 2 3) (-2 1 7) (0 0 0)).
	c := a + b.
	self should: [ c numberOfRows = 3].
	self should: [ c numberOfColumns = 3].
	self should: [ ((c rowAt: 1) at: 1) = 12].
	self should: [ ((c rowAt: 1) at: 2) = 14].
	self should: [ ((c rowAt: 1) at: 3) = 16].
	self should: [ ((c rowAt: 2) at: 1) = 19].
	self should: [ ((c rowAt: 2) at: 2) = 23].
	self should: [ ((c rowAt: 2) at: 3) = 30].
	self should: [ ((c rowAt: 3) at: 1) = 31].! !

!DhbNumericalMethodsTestCase methodsFor: 'estimation' stamp: 'DanielUber 5/22/2012 07:50:06.294'!
testTTest

	| accC accMM confidenceLevel|
	accC := DhbStatisticalMoments new.
	#( 5.56 5.89 4.66 5.69 5.34 4.79 4.80 7.86 3.64 5.70 )
		do: [ :x | accC accumulate: x].
	accMM := DhbStatisticalMoments new.
	#( 7.48 6.75 3.77 5.71 7.25 4.73 6.23 5.60 5.94 4.58 )
		do: [ :x | accMM accumulate: x].
	confidenceLevel := accC tConfidenceLevel: accMM.
	self should: [ (accC average - 5.393) abs < 0.000000001].
	self should: [ (accC standardDeviation - 1.0990809292) abs < 0.000000001].
	self should: [ (accMM average - 5.804) abs < 0.000000001].
	self should: [ (accMM standardDeviation - 1.19415428) abs < 0.000000001].
	self should: [ (confidenceLevel - 56.6320739989) abs < 0.000000001].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 4/15/2014 09:56'!
testVectorMatrixOperation
	"Code Example 8.1"
	| a u v |
	a := DhbMatrix rows: #( ( 1 0 1) (-1 -2 3)).
	u := #(1 2 3) asDhbVector.
	v := a * u.
	self should: [ v size = 2].
	self should: [ (v at: 1) = 4].
	self should: [ (v at: 2) = 4].! !

!DhbNumericalMethodsTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 4/15/2014 09:56'!
testVectorTransposeMatrixOperation
	"Code Example 8.1"
	| c v w |
	c := DhbMatrix rows: #( ( 6 8 10) (18 14 4)).
	v := #(4 4) asDhbVector.
	w := c transpose * v.
	self should: [ w size = 3].
	self should: [ (w at: 1) = 96].
	self should: [ (w at: 2) = 88].
	self should: [ (w at: 3) = 56].! !

TestCase subclass: #DhbPolynomialTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbPolynomialTestCase methodsFor: 'comparing' stamp: 'DanielUber 5/31/2012 11:12'!
testIsZero
	| p1 p2 |
	p1 := DhbPolynomial coefficients: #(0 0 0 0 0).
	p2 := DhbPolynomial coefficients: #(0 0 0 0 1 0 0 0).
	self should: [p1 isZero ].
	self shouldnt: [p2 isZero ].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:11'!
testPolynomialAddition

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) + ( DhbPolynomial coefficients: #(-3 7 2 1)).
	self should: [ (polynomial at: 0) = -1].
	self should: [ (polynomial at: 1) = 4].
	self should: [ (polynomial at: 2) = 3].
	self should: [ (polynomial at: 3) = 1].
	self should: [ (polynomial at: 4) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:11'!
testPolynomialDerivative
	"Code example 2.3"
	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(-3 7 2 1)) derivative.
	self should: [ (polynomial at: 0) = 7].
	self should: [ (polynomial at: 1) = 4].
	self should: [ (polynomial at: 2) = 3].
	self should: [ (polynomial at: 3) = 0].
	self should: [ (polynomial at: 4) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:11'!
testPolynomialDivision

	| pol1 pol2 polynomial |
	pol1 := DhbPolynomial coefficients: #(2 -3 1).
	pol2 := DhbPolynomial coefficients: #(-6 23 -20 3 -1 1).
	polynomial := pol2 / pol1.
	self should: [ (polynomial at: 0) = -3].
	self should: [ (polynomial at: 1) = 7].
	self should: [ (polynomial at: 2) = 2].
	self should: [ (polynomial at: 3) = 1].
	self should: [ (polynomial at: 4) = 0].
	self should: [ (polynomial at: 5) = 0].
	self should: [ (polynomial at: 6) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/31/2012 09:11'!
testPolynomialDivisionBug
	"identify an error when trying to create a zero dividend"
	| pol1 pol2 polynomial |
	pol1 := DhbPolynomial coefficients: #(2 -3 1).
	pol2 := DhbPolynomial coefficients: #(-6 23 -20 3 -1 1).
	self shouldnt: [polynomial := pol1 / pol2] raise: Error.
	! !

!DhbPolynomialTestCase methodsFor: 'arithmetic' stamp: 'DanielUber 5/23/2012 14:10'!
testPolynomialDoubleDispatch
	| n p |
	n := 3.2.
	p := DhbPolynomial coefficients: #(1 0 1).
	self should: [(p + n) = (n + p)].
	self should: [(p * n )= (n * p)].
	n := 3.
	self should: [(p + n) = (n + p)].
	self should: [(p * n )= (n * p)].
	self shouldnt: [(n - p) = (p - n)].
	self should: [(n - p) = ((p - n) * -1) ].
	n := ( 1 / 2).
	self should: [n + p = (p + n)].
	self should: [n - p = (p - n)  negated].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:11'!
testPolynomialEvaluation
	"Code example 2.2"
	| polynomial |
	polynomial := DhbPolynomial coefficients: #(2 -3 1).
	self should: [0 = (polynomial value: 1)].! !

!DhbPolynomialTestCase methodsFor: 'comparing' stamp: 'DanielUber 5/23/2012 14:13'!
testPolynomialHash
	"polynomial hash is hash of coefficient array"
	| p1 p2 p3 |
	p1 := DhbPolynomial coefficients: #(1 0 1).
	p2 := DhbPolynomial coefficients: #(1 0 1).
	p3 := DhbPolynomial coefficients: #(1 1 1).
	self should: [p1 hash = p1 hash].
	self should: [p2 hash = p2 hash].
	self shouldnt: [p3 hash = p1 hash].
	self shouldnt: [p3 hash = p2 hash].
	p3 := p3 - (DhbPolynomial  coefficients: #(0 1)).
	self should: [p3 hash = p2 hash].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:13'!
testPolynomialIntegral
	"Code example 2.3"
	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(-3 7 2 1)) integral.
	self should: [ (polynomial at: 0) = 0].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = (7/2)].
	self should: [ (polynomial at: 3) = (2/3)].
	self should: [ (polynomial at: 4) = (1/4)].
	self should: [ (polynomial at: 5) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:11'!
testPolynomialIntegralWithConstant
	"Code example 2.3"
	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(-3 7 2 1)) integral: 5.
	self should: [ (polynomial at: 0) = 5].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = (7/2)].
	self should: [ (polynomial at: 3) = (2/3)].
	self should: [ (polynomial at: 4) = (1/4)].
	self should: [ (polynomial at: 5) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:11'!
testPolynomialMultiplication
	"Code example 2.3"
	| pol1 pol2 polynomial |
	pol1 := DhbPolynomial coefficients: #(2 -3 1).
	pol2 := DhbPolynomial coefficients: #(-3 7 2 1).
	polynomial := pol1 * pol2.
	self should: [ (polynomial at: 0) = -6].
	self should: [ (polynomial at: 1) = 23].
	self should: [ (polynomial at: 2) = -20].
	self should: [ (polynomial at: 3) = 3].
	self should: [ (polynomial at: 4) = -1].
	self should: [ (polynomial at: 5) = 1].
	self should: [ (polynomial at: 6) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:12'!
testPolynomialNumberAddition

	| polynomial |
	polynomial := 2 + ( DhbPolynomial coefficients: #(2 -3 1)).
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = 1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:13'!
testPolynomialNumberAdditionInverse

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) + 2.
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = 1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:12'!
testPolynomialNumberDivision

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) / 2.
	self should: [ (polynomial at: 0) = 1].
	self should: [ (polynomial at: 1) = (-3 / 2)].
	self should: [ (polynomial at: 2) = (1/2)].
	self should: [ (polynomial at: 3) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:12'!
testPolynomialNumberMultiplication

	| polynomial |
	polynomial := 2 * ( DhbPolynomial coefficients: #(2 -3 1)).
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -6].
	self should: [ (polynomial at: 2) = 2].
	self should: [ (polynomial at: 3) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:12'!
testPolynomialNumberMultiplicationInverse

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) * 2.
	self should: [ (polynomial at: 0) = 4].
	self should: [ (polynomial at: 1) = -6].
	self should: [ (polynomial at: 2) = 2].
	self should: [ (polynomial at: 3) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:12'!
testPolynomialNumberSubtraction

	| polynomial |
	polynomial := 2 - ( DhbPolynomial coefficients: #(2 -3 1)).
	self should: [ (polynomial at: 0) = 0].
	self should: [ (polynomial at: 1) = 3].
	self should: [ (polynomial at: 2) = -1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:11'!
testPolynomialNumberSubtractionInverse

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) - 2.
	self should: [ (polynomial at: 0) = 0].
	self should: [ (polynomial at: 1) = -3].
	self should: [ (polynomial at: 2) = 1].
	self should: [ (polynomial at: 3) = 0].! !

!DhbPolynomialTestCase methodsFor: 'printing' stamp: 'DanielUber 5/23/2012 14:12'!
testPolynomialPrintOn
	| poly|
	poly := DhbPolynomial coefficients: #( 1 0 1).
	self should: [poly printString = '1 +  X^2'].
	poly := DhbPolynomial coefficients: #(1).
	self should: [poly printString = '1'].
! !

!DhbPolynomialTestCase methodsFor: 'iterative algorithms' stamp: 'DanielUber 5/23/2012 14:13'!
testPolynomialRoots
	"Code Example 5.5"
	| polynomial roots |
	polynomial := DhbPolynomial coefficients: #( -10 -13 -2 1).
	roots := polynomial roots asSortedCollection asArray.
	self should: [ roots size = 3].
	self should: [ ( (roots at: 1) + 2) abs < DhbFloatingPointMachine new defaultNumericalPrecision].
	self should: [ ( (roots at: 2) + 1) abs < DhbFloatingPointMachine new defaultNumericalPrecision].
	self should: [ ( (roots at: 3) - 5) abs < DhbFloatingPointMachine new defaultNumericalPrecision].! !

!DhbPolynomialTestCase methodsFor: 'function evaluation' stamp: 'DanielUber 5/23/2012 14:13'!
testPolynomialSubtraction

	| polynomial |
	polynomial := ( DhbPolynomial coefficients: #(2 -3 1)) - ( DhbPolynomial coefficients: #(-3 7 2 1)).
	self should: [ (polynomial at: 0) = 5].
	self should: [ (polynomial at: 1) = -10].
	self should: [ (polynomial at: 2) = -1].
	self should: [ (polynomial at: 3) = -1].
	self should: [ (polynomial at: 4) = 0].! !

TestCase subclass: #DhbStatisticsBugs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbStatisticsBugs methodsFor: 'as yet unclassified' stamp: 'wernerkassens 10/30/2014 11:22'!
testDhbFisherTippet
|d|
d:=DhbFisherTippettDistribution new.
self assert: ((d integralFrom: -10 to: -0.1)closeTo: (d distributionValue: -0.1)).
self assert:((d integralUpTo: -0.1)closeTo: (d distributionValue: -0.1)).
self assert:((d integralFrom: -10 to: 0.1)closeTo: (d distributionValue: 0.1)).

! !

!DhbStatisticsBugs methodsFor: 'as yet unclassified' stamp: 'wernerkassens 10/30/2014 12:17'!
testDhbProbabilityDensityWithUnknownDistribution
|d|
d:=DhbLogNormalDistribution new.
self assert: ((d distributionValue: 1)closeTo: 0.5).
self assert:((d distributionValue: 3.602)closeTo:  0.9).
self assert:((d acceptanceBetween: 1 and: 3.602)closeTo: 0.4).

! !

TestCase subclass: #DhbSymmetricMatrixBug
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbSymmetricMatrixBug methodsFor: 'tests' stamp: 'WernerKassens 4/8/2014 18:54'!
testCrlInverse
|m i|
m:=DhbSymmetricMatrix rows: #((2 2 3)(2 4 5)(3 5 6)).
i:=m crlInverse.
self assert: (m*i=( DhbSymmetricMatrix identity: 3)).
"the funny result of m*i with the bug is this matrix:
a DhbVector(1 0 -24)
a DhbVector(0 1 -40)
a DhbVector(2 4 -51)" 
! !

!DhbSymmetricMatrixBug methodsFor: 'tests' stamp: 'WernerKassens 4/9/2014 17:19'!
testEqual
"= should work like in a SequenceableCollection  eg a DhbVector and a Array can be equal"
|a b|
a:=DhbMatrix rows: #((2 2 3)(2 4 5)(3 5 6)).
b:=a asSymmetricMatrix .
self assert: (a=b).
self assert: (b=a).
self assert: (a hash=b hash).
a:=DhbMatrix rows: #((1 2 3)(2 4 5)(3 5 6)).
self assert: (a~=b).
self assert: (b~=a).
self assert: (a hash~=b hash).
a:=a asSymmetricMatrix .
self assert: (a~=b).
a:=2 .
self assert: (b~=a).
a:='s' .
self assert: (b~=a).



! !

!DhbSymmetricMatrixBug methodsFor: 'tests' stamp: 'WernerKassens 4/13/2014 11:26'!
testInversePureLUP
|m i|
m:=DhbSymmetricMatrix rows: #((2 2 3)(2 4 5)(3 5 6)).
"this raises an error"
self shouldnt: [i:=m inversePureLUP]raise: Error.
self assert: (m*i=( DhbSymmetricMatrix identity: 3)).
"there is no(!!) bug in inversePureCRL, but it is nowhere tested i think, hence i just added this here for the sake of simplicity (and because i simplified inversePureCRL)"
i:=m inversePureCRL .
self assert: (m*i=( DhbSymmetricMatrix identity: 3)).
! !

TestCase subclass: #DhbVectorBug
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbVectorBug methodsFor: 'tests' stamp: 'WernerKassens 4/15/2014 09:57'!
testTensorProduct
|a b c|
a:=#(1 2 3)asDhbVector.
b:=#(2 2 3)asDhbVector.
c:=a tensorProduct: b.
self deny: (c transpose = c). "hence c is not symmetric, but this does not work:"
self deny: (c isSymmetric) .
"furthermore this is completely permissible:"
b:=#(1 2 3 4)asDhbVector.
self shouldnt: [c:=a tensorProduct: b] raise: Error.
self assert:((b tensorProduct: a)=c transpose ).

! !

TestCase subclass: #DhbVectorTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Tests-DHB-Numerical'!

!DhbVectorTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 6/4/2012 08:01'!
testScalarProduct
	| u v |
	u := DhbVector new: 3.
	v := DhbVector new: 3.
	u at: 1 put: 1; at: 2 put: 1; at: 3 put: 1.
	v at: 1 put: 0; at: 2 put: 2; at: 3 put: 0.
	self should: [(u scalarProduct: v) = 2].
	v at:  1 put: 2.
	self should: [(u scalarProduct: v) = 4].! !

!DhbVectorTestCase methodsFor: 'linear algebra' stamp: 'DanielUber 6/4/2012 08:00'!
testScalarProductIsCommutative
	| u v |
	u := DhbVector new: 3.
	v := DhbVector new: 3.
	u at: 1 put: 1; at: 2 put: 1; at: 3 put: 1.
	v at: 1 put: 0; at: 2 put: 2; at: 3 put: 0.
	self should: [(u scalarProduct: v ) = (v scalarProduct: u)].
! !

!DhbVectorTestCase methodsFor: 'arithmetic' stamp: 'WernerKassens 4/15/2014 09:56'!
testVectorNegate
	"negate modifies the vector. negated makes a new vector with inverse coefficients. "
	| v |
	v := #(1 2 3) asDhbVector .
	self should: [(v negate at: 1)= -1].
	self should: [2 = (v negate at: 2)].
	self should: [-2 = (v negated at: 2)].! !

!DhbVectorTestCase methodsFor: 'information' stamp: 'WernerKassens 4/15/2014 09:57'!
testVectorNorm
	| v1 v2 |
	v1 := #(1 0) asDhbVector.
	self should: [v1 norm = 1].
	v2 := #(0 1) asDhbVector.
	self should: [v2 norm = 1].
	self shouldnt: [(v1 + v2) norm = 1].
	"normalized norm may differ from exactly one due to floating point operations"
	self should: [(v1 + v2) normalized norm equalsTo:  1].! !

!DhbVectorTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 4/15/2014 09:56'!
testVectorOperation
	"Code Example 8.1"
	| u v w |
	u := #(1 2 3) asDhbVector.
	v := #(3 4 5) asDhbVector.
	w := 4 * u + (3 * v).
	self should: [ w size = 3].
	self should: [ (w at: 1) = 13].
	self should: [ (w at: 2) = 20].
	self should: [ (w at: 3) = 27].! !

!DhbVectorTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 4/15/2014 09:56'!
testVectorOperationInverse

	| u v w |
	u := #(1 2 3) asDhbVector.
	v := #(3 4 5) asDhbVector.
	w := v * 4 - (3 * u).
	self should: [ w size = 3].
	self should: [ (w at: 1) = 9].
	self should: [ (w at: 2) = 10].
	self should: [ (w at: 3) = 11].! !

!DhbVectorTestCase methodsFor: 'linear algebra' stamp: 'WernerKassens 4/15/2014 09:56'!
testVectorProduct
	"Code Example 8.1"
	| u v |
	u := #(1 2 3) asDhbVector.
	v := #(3 4 5) asDhbVector.
	self should: [ u * v = 26].! !
